@@ -6,7 +6,7 @@ use strict;
 use 5.008003;
 no warnings 'utf8';
 
-our $VERSION = '1.016';
+our $VERSION = '1.017';
 
 
 
@@ -16,13 +16,14 @@ App::DBBrowser::DB - Database plugin documentation.
 
 =head1 VERSION
 
-Version 1.016
+Version 1.017
+
 
 =head1 DESCRIPTION
 
-A database plugin provides the database specific methods. C<App::DBBrowser> considers a module whose name matches the
-regex pattern C</^App::DBBrowser::DB::[\w_]+\z/> and which is located in one of the C<@INC> directories as a database
-plugin. Plugins with the name C<App::DBBrowser::DB::$database_driver> should be for general use of C<$database_driver>
+A database plugin provides the database specific methods. C<App::DBBrowser> considers a module whose name matches
+C</^App::DBBrowser::DB::[^:']+\z/> and which is located in one of the C<@INC> directories as a database plugin.
+Plugins with the name C<App::DBBrowser::DB::$database_driver> should be for general use of C<$database_driver>
 databases.
 
 The user can add an installed database plugin to the available plugins in the option menu (C<db-browser -h>) by
@@ -32,14 +33,17 @@ A suitable database plugin provides the methods named in this documentation.
 
 Column names passed as arguments to plugin methods are already quoted with the C<DBI> C<quote_identifier> method.
 
+
 =head1 PLUGIN API VERSION
 
-This documentation describes the plugin API version C<1.4>.
+This documentation describes the plugin API version C<1.5>.
+
+Supported plugin API versions: C<1.4> and C<1.5>.
 
-Supported plugin API versions: C<1.4>.
 
 =head1 METHODS
 
+
 =head2 new
 
 The constructor method.
@@ -69,10 +73,6 @@ The object.
 
 sub new {
     my ( $class, $info, $opt ) = @_;
-    if ( $info->{db_plugin} !~ /^[\w_]+\z/ ) {
-        die "Invalid character in the DB plugin name: $info->{db_plugin}";
-    }
-
     my $db_module = 'App::DBBrowser::DB::' . $info->{db_plugin};
     eval "require $db_module" or die $@;
 
@@ -99,11 +99,13 @@ sub new {
     bless { Plugin => $plugin }, $class;
 }
 
+
 sub message_method_undef_return {
     my ( $self, $method ) = @_;
     return sprintf '%s method %s: no return value', ref $self->{Plugin}, $method;
 }
 
+
 sub debug {
     my ( $self, $dbh, $info, $opt, $db_opt ) = @_;
     return if ! $self->{Plugin}->can( 'debug' );
@@ -111,6 +113,7 @@ sub debug {
 }
 
 
+
 =head2 plugin_api_version
 
 =over
@@ -239,7 +242,6 @@ I<Fields> and I<Login Data>.
                           option "Login Data"  =>  $connect_parameter->{read_arg}
                                                =>  $connect_parameter->{keep_secret}
 
-
 =cut
 
 sub read_arguments {
@@ -516,6 +518,36 @@ sub get_table_names {
 
 
 
+=head2 primary_key_auto
+
+=over
+
+=item Arguments
+
+none
+
+=item return
+
+The primary-key-autoincrement statement.
+
+=back
+
+Example for the database driver C<Pg>:
+
+    sub primary_key_auto {
+        return "SERIAL PRIMARY KEY";
+    }
+
+=cut
+
+sub primary_key_auto {
+    my ( $self ) = @_;
+    return if ! $self->{Plugin}->can( 'primary_key_auto' ); #
+    return $self->{Plugin}->primary_key_auto();
+}
+
+
+
 =head2 column_names_and_types
 
 =over
