@@ -6,7 +6,7 @@ use strict;
 use 5.008003;
 no warnings 'utf8';
 
-our $VERSION = '1.016';
+our $VERSION = '1.017';
 
 use File::Basename        qw( basename fileparse );
 use File::Spec::Functions qw( catfile );
@@ -56,7 +56,7 @@ sub defaults {
             binary_filter        => 0,
         },
         insert => {
-            input_modes          => [ 'Cols', 'Multi-row', 'File' ],
+            input_modes          => [ 'Cols', 'Rows', 'Multi-row', 'File' ],
             parse_mode           => 0,
             file_encoding        => 'UTF-8',
             max_files            => 15,
@@ -74,6 +74,9 @@ sub defaults {
         # split:
             i_r_s                => '\n',
             i_f_s                => ',',
+        # create table defaults:
+            id_col_name          => 'ID_a',
+            default_data_type    => 'TEXT',
         }
     };
     return $defaults                   if ! $section;
@@ -86,12 +89,13 @@ sub __sub_menus_insert {
     my ( $self, $group ) = @_;
     my $sub_menus_insert = {
         main_insert => [
-            { name => 'input_modes',       text => "- Read",          section => 'insert' },
-            { name => 'parse_mode',        text => "- Parse-mode",    section => 'insert' },
-            { name => '_module_Text_CSV',  text => "- conf T::CSV",   section => 'insert' },
-            { name => '_parse_with_split', text => "- conf 'split'",  section => 'insert' },
-            { name => 'file_encoding',     text => "- File Encoding", section => 'insert' },
-            { name => 'max_files',         text => "- File History",  section => 'insert' },
+            { name => 'input_modes',           text => "- Read",          section => 'insert' },
+            { name => 'parse_mode',            text => "- Parse-mode",    section => 'insert' },
+            { name => '_module_Text_CSV',      text => "- conf T::CSV",   section => 'insert' },
+            { name => '_parse_with_split',     text => "- conf 'split'",  section => 'insert' },
+            { name => 'file_encoding',         text => "- File Encoding", section => 'insert' },
+            { name => 'max_files',             text => "- File History",  section => 'insert' },
+            { name => 'create_table_defaults', text => "- Create-table",  section => 'insert' },
         ],
         _module_Text_CSV => [
             { name => '_csv_char',    text => "- *_char attributes", section => 'insert' },
@@ -212,6 +216,14 @@ sub __config_insert {
                 my $prompt = '"Max file history"';
                 $self->__opt_number_range( $opt_type, $section, $option, $prompt, $digits );
             }
+            elsif ( $option eq 'create_table_defaults' ) {
+                my $items = [
+                    { name => 'id_col_name',       prompt => "Default ID col name  " },
+                    { name => 'default_data_type', prompt => "Default data type" },
+                ];
+                my $prompt = 'Create-table defaults';
+                $self->__group_readline( $opt_type, $section, $items, $prompt );
+            }
             else { die "Unknown option: $option" }
         }
     }
@@ -258,7 +270,7 @@ sub __menus {
 }
 
 
-sub set_options {
+sub __set_options {
     my ( $self ) = @_;
     my $group = 'main';
     my $backup_old_idx = 0;
@@ -305,6 +317,7 @@ sub set_options {
             }
             my $option = $idx <= $#pre ? $pre[$idx] : $menu->[$idx - @pre]{name};
             if ( $option eq 'config_insert' ) {
+                $backup_old_idx = $old_idx;
                 $self->__config_insert();
                 $old_idx = $backup_old_idx;
                 $group = 'main';
@@ -344,7 +357,7 @@ sub set_options {
                 next OPTION;
             }
             elsif ( $option eq '_db_defaults' ) {
-                $self->database_setting();
+                $self->__database_setting();
                 next OPTION;
             }
             my $opt_type = 'opt';
@@ -358,7 +371,7 @@ sub set_options {
                 }
                 my $prompt = 'Choose DB plugins:';
                 $self->__opt_choose_a_list( $opt_type, $section, $option, [ sort keys %installed_db_driver ], $prompt );
-                $self->read_db_config_files();
+                $self->__read_db_config_files();
             }
             elsif ( $option eq 'tab_width' ) {
                 my $digits = 3;
@@ -510,7 +523,7 @@ sub __opt_choose_dirs {
 }
 
 
-sub database_setting {
+sub __database_setting {
     my ( $self, $db ) = @_;
     my $changed = 0;
     SECTION: while ( 1 ) {
@@ -723,7 +736,7 @@ sub __write_config_files {
     }
     my $auxil = App::DBBrowser::Auxil->new( $self->{info} );
     my $file_name = $self->{info}{config_generic};
-    $auxil->write_json( sprintf( $fmt, $file_name ), $tmp  );
+    $auxil->__write_json( sprintf( $fmt, $file_name ), $tmp  );
 }
 
 
@@ -744,19 +757,19 @@ sub __write_db_config_files {
     my $auxil = App::DBBrowser::Auxil->new( $self->{info} );
     for my $section ( keys %$tmp ) {
         my $file_name =  $section;
-        $auxil->write_json( sprintf( $fmt, $file_name ), $tmp->{$section}  );
+        $auxil->__write_json( sprintf( $fmt, $file_name ), $tmp->{$section}  );
     }
 }
 
 
-sub read_config_files {
+sub __read_config_files {
     my ( $self ) = @_;
     $self->{opt} = $self->defaults();
     my $fmt = $self->{info}{conf_file_fmt};
     my $auxil = App::DBBrowser::Auxil->new( $self->{info} );
     my $file =  sprintf( $fmt, $self->{info}{config_generic} );
     if ( -f $file && -s $file ) {
-        my $tmp = $auxil->read_json( $file );
+        my $tmp = $auxil->__read_json( $file );
         for my $section ( keys %$tmp ) {
             for my $option ( keys %{$tmp->{$section}} ) {
                 $self->{opt}{$section}{$option} = $tmp->{$section}{$option} if exists $self->{opt}{$section}{$option};
@@ -767,14 +780,14 @@ sub read_config_files {
 }
 
 
-sub read_db_config_files {
+sub __read_db_config_files {
     my ( $self ) = @_;
     my $fmt = $self->{info}{conf_file_fmt};
     my $auxil = App::DBBrowser::Auxil->new( $self->{info} );
     for my $db_plugin ( @{$self->{opt}{G}{db_plugins}} ) {
         my $file = sprintf( $fmt, $db_plugin );
         if ( -f $file && -s $file ) {
-            my $tmp = $auxil->read_json( $file );
+            my $tmp = $auxil->__read_json( $file );
             for my $conf_sect ( keys %$tmp ) {
                 my $section = $db_plugin . ( $conf_sect =~ /^\*\Q$db_plugin\E\z/ ? '' : '_' . $conf_sect );
                 for my $option ( keys %{$tmp->{$conf_sect}} ) {
